package com.gic.enterprise.service.outer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.entity.TabAppletDecorationConfig;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.AppletDataAssetAreaDTO;
import com.gic.enterprise.dto.AppletDecorationConfigDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.AppletDecorationConfigApiService;
import com.gic.enterprise.service.AppletDecorationConfigService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletCardRelationDTO;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;

@Service("appletDecorationConfigApiService")
public class AppletDecorationConfigApiServiceImpl implements AppletDecorationConfigApiService {
    private static final Logger LOGGER = LogManager.getLogger(AppletDecorationConfigApiServiceImpl.class);
    @Autowired
    private AppletDecorationConfigService appletDecorationConfigService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @Override
    public ServiceResponse<Integer> save(AppletDecorationConfigDTO dto) {

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        //
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(dto.getAppId(), dto.getEnterpriseId());
        if (record != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不得重复配置");
        }

        if (dto.getGpsDecisionCard() == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        }

        //默认选中和主卡关联的店铺，不可编辑
        ServiceResponse<ShopDTO> shopResult = getMainCardStore(dto.getAppId(), dto.getEnterpriseId());
        if (!shopResult.isSuccess()) {
            return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
        }
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        ShopDTO shopDTO = shopResult.getResult();
        appletDataAssetArea.setMallOrder(shopDTO.getId().toString());
        appletDataAssetArea.setMallOrderName(shopDTO.getName());

        //默认配置信息
        ServiceResponse<Long> mainCardResult = getMainCard(dto.getAppId(), dto.getEnterpriseId());
        if (!mainCardResult.isSuccess()) {
            return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
        }
        String mainCardId = mainCardResult.getResult().toString();
        //签到 系统默认主卡，不可编辑
        appletDataAssetArea.setSignIn(mainCardId);
        //会员成就 系统默认主卡，不可编辑
        appletDataAssetArea.setMemberAchievement(mainCardId);
        //会员任务 系统默认主卡，不可编辑
        appletDataAssetArea.setMemberTask(mainCardId);
        //数据资产设置
        dto.setDataAssetArea(JSON.toJSONString(appletDataAssetArea));
        return ServiceResponse.success(appletDecorationConfigService.save(dto));
    }

    @Override
    public ServiceResponse<String> edit(AppletDecorationConfigDTO dto) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getById(dto.getConfigId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "装修配置ID错误，查无数据");
        }
        dto.setAppId(record.getAppId());
        dto.setEnterpriseId(record.getEnterpriseId());

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        if (dto.getGpsDecisionCard() == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        }
        AppletDataAssetAreaDTO old = JSON.parseObject(record.getDataAssetArea(), AppletDataAssetAreaDTO.class);
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        appletDataAssetArea.setMallOrder(old.getMallOrder());
        //默认配置信息
        ServiceResponse<Long> mainCardResult = getMainCard(dto.getAppId(), dto.getEnterpriseId());
        if (!mainCardResult.isSuccess()) {
            return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
        }
        String mainCardId = mainCardResult.getResult().toString();
        //签到 系统默认主卡，不可编辑
        appletDataAssetArea.setSignIn(mainCardId);
        //会员成就 系统默认主卡，不可编辑
        appletDataAssetArea.setMemberAchievement(mainCardId);
        //会员任务 系统默认主卡，不可编辑
        appletDataAssetArea.setMemberTask(mainCardId);
        //数据资产设置
        dto.setDataAssetArea(JSON.toJSONString(appletDataAssetArea));
        appletDecorationConfigService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<AppletDecorationConfigDTO> getDetailByAppId(String appId, Integer enterpriseId) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(appId, enterpriseId);
        if (record == null) {
            //默认配置信息
            ServiceResponse<Long> mainCardResult = getMainCard(appId, enterpriseId);
            if (!mainCardResult.isSuccess()) {
                return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
            }
            //默认选中和主卡关联的店铺，不可编辑
            ServiceResponse<ShopDTO> shopResult = getMainCardStore(appId, enterpriseId);
            if (!shopResult.isSuccess()) {
                return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
            }
            ShopDTO shopDTO = shopResult.getResult();

            Long mainCardId = mainCardResult.getResult();
            AppletDecorationConfigDTO result = new AppletDecorationConfigDTO();
            //数据资产配置信息
            AppletDataAssetAreaDTO dataAssetArea = new AppletDataAssetAreaDTO();
            dataAssetArea.setOrder(mainCardId.toString());
            dataAssetArea.setCoupon(mainCardId.toString());
            dataAssetArea.setIntegral(mainCardId.toString());
            dataAssetArea.setMallOrder(shopDTO.getId().toString());
            dataAssetArea.setMallOrderName(shopDTO.getName());
            //会员中心页面卡范围
            result.setMemberCardArea(mainCardId.toString());
            //数据资产对象
            result.setAppletDataAssetArea(dataAssetArea);
            //数据资产json字符串
            result.setDataAssetArea(JSON.toJSONString(dataAssetArea));
            //GPS智能判定卡 默认关闭
            result.setGpsDecisionCard(0);
            result.setEnterpriseId(enterpriseId);
            result.setAppId(appId);
            //开卡认证关联
            result.setOpenCardAuth(mainCardId.toString());
            return ServiceResponse.success(result);
        }
        AppletDecorationConfigDTO dto = EntityUtil.changeEntityNew(AppletDecorationConfigDTO.class, record);
        AppletDataAssetAreaDTO old = JSON.parseObject(record.getDataAssetArea(), AppletDataAssetAreaDTO.class);
        dto.setAppletDataAssetArea(old);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<Object> listAppletCard(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardRelationDTO>> result = getMemberCardByAppId(appId, enterpriseId);
        if (result.isSuccess()) {
            return ServiceResponse.success(result.getResult());
        }
        return ServiceResponse.failure(result.getCode(), result.getMessage());
    }

    private ServiceResponse valueValid(AppletDecorationConfigDTO dto) {
        String appId = dto.getAppId();
        Integer enterpriseId = dto.getEnterpriseId();
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        if (StringUtils.isBlank(appId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序ID不能为空");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (appletDataAssetArea == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产配置不能为空");
        }
        //校验参数数值是否非法
        ServiceResponse<Map<String, Object>> appletCardMapResult = memberCardToMap(appId, enterpriseId);
        if (!appletCardMapResult.isSuccess()) {
            return ServiceResponse.failure(appletCardMapResult.getCode(), appletCardMapResult.getMessage());
        }
        Map<String, Object> appletCardMap = appletCardMapResult.getResult();
        //需要校验的数值集合
        Map<String, String> needValidMap = new HashMap<>(16);
        needValidMap.put("会员中心页面卡范围", dto.getMemberCardArea());
        needValidMap.put("开卡认证关联", dto.getOpenCardAuth());
        //数据资产
        needValidMap.put("数据资产-积分", appletDataAssetArea.getIntegral());
        needValidMap.put("数据资产-卡券包", appletDataAssetArea.getCoupon());
        needValidMap.put("数据资产-零售订", appletDataAssetArea.getOrder());

        ServiceResponse valueValid;
        for (Map.Entry<String, String> entry : needValidMap.entrySet()) {
            valueValid = validParamValue(entry.getKey(), entry.getValue(), appletCardMap);
            if (!valueValid.isSuccess()) {
                return ServiceResponse.failure(valueValid.getCode(), valueValid.getMessage());
            }
        }

        Integer gpsDecisionCard = dto.getGpsDecisionCard();
        if (gpsDecisionCard == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        } else {
            if (gpsDecisionCard != 1 && gpsDecisionCard != 0) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "GPS智能判定卡数值非法");
            }
        }
        return ServiceResponse.success();
    }

    private ServiceResponse<Map<String, Object>> memberCardToMap(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardRelationDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        Map<String, Object> result = new HashMap<>(16);
        List<AppletCardRelationDTO> list =  appletCardResult.getResult();
        for (AppletCardRelationDTO card : list) {
            result.put(card.getCardConfigId().toString(), "1");
        }
        return ServiceResponse.success(result);
    }

    private ServiceResponse validParamValue(String key, String value, Map<String, Object> appletCardMap) {
        if (StringUtils.isBlank(value)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),  key + "不能为空");
        }
        String[] arr = value.split(",");
        for (String str : arr) {
            if (!appletCardMap.containsKey(str)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), key + "数值非法");
            }
        }
        return ServiceResponse.success();
    }

    /**
     * 查询会员卡列表
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<List<AppletCardRelationDTO>> getMemberCardByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        List<Long> list = new ArrayList<>(1);
        list.add(appletConfigId);
        ServiceResponse<List<AppletCardRelationDTO>> appletCardResult = appletsConfigApiService
                .queryAppletCardRelation(list, enterpriseId);
        if (appletCardResult.isSuccess()) {
            return ServiceResponse.success(appletCardResult.getResult());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletCardResult.getMessage());
    }

    private ServiceResponse<Long> getMainCard(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardRelationDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        List<AppletCardRelationDTO> list = appletCardResult.getResult();
        for (AppletCardRelationDTO dto : list) {
            if (dto.getMainCard()) {
                return ServiceResponse.success(dto.getCardConfigId());
            }
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有主卡");
    }

    /**
     * 获取和主卡关联的店铺
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<ShopDTO> getMainCardStore(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        //shopType 1:微商城 2：积分
        ServiceResponse<ShopDTO> shopResult = shopApiService.getShopByAppletId(appletConfigId, enterpriseId, 1);
        if (shopResult.isSuccess()) {
            return ServiceResponse.success(shopResult.getResult());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), shopResult.getMessage());
    }

    /**
     * 根据appId查询小程序配置ID
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<Long> getAppletConfigIdByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<AppletsConfigSimpleOutputDTO> appletResult = appletsConfigApiService
                .selectAppletsConfigIdByAppId(appId, enterpriseId);
        if (appletResult.isSuccess()) {
            return ServiceResponse.success(appletResult.getResult().getAppletConfigId());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletResult.getMessage());
    }

}
